/*
THR_TEST.PRC

written by:

Bruce E. Hansen
Department of Economics
Social Science Building
University of Wisconsin
Madison, WI 53706-1393
bhansen@ssc.wisc.edu
http://www.ssc.wisc.edu/~bhansen/


This is a GAUSS procedure.
It computes a test for a threshold in linear regression
under homoskedasticity.  The procedure takes the form

{f_test,p_value} = thr_test(dat,yi,xi,qi,trim_per,rep)

The inputs are:
dat      = data matrix (nxk)
yi       = index of dependent (y) variable, e.g. yi = 1
xi       = indexes of independent (x) variables, e.g. xi = 2|3
qi       = index of threshold (q) variable, e.g. qi = 4
trim_per = percentage of sample to trim from ends, e.g. trim_per = .15
rep      = number of bootstrap replications, e.g. rep = 1000

Output:
f_test   = Value of Maximal (Quandt) F-statistic
p_value  = Bootstrap p-value

Notes: 
(1)    Do not include a constant in the independent variables,
       the program automatically adds an intercept to the regression.

(2)    The program stores the sequential design matrices in order
       to speed up the bootstrap computations.  It is possible that if
       your dataset is very large, this will exceed your computer RAM
       memory.  If so, program will crash, and the message
              error G0030: Insufficient Workspace Memory
       will be displayed.  If a more RAM is not available, 
       change the switch _quick in the program to read "_quick=0;".  
       This switch requires the bootstrap to re-calculate the design
       matrices for each bootstrap replication, which requires less memory,
       but somewhat more computer time.

Example:
If the nxk matrix "dat" contains the dependent variable in the first
column, the independent variables in the second through tenth columns,
and the threshold variable in the fifth:

xi = 2|3|4|5|6|7|8|9|10;
{f_test,p_value} = thr_test(dat,1,xi,5,.15,1000);



*/

/*******************************************************************/

proc (2) = thr_test(dat,yi,xi,qi,trim_per,rep);
local n,q,y,x,k,mi,e,ee,xe,qs,qn,sn,mm,sume,mmistore,r,
smax,sig,ftest,fboot,j,pv,cr,lr,clr,mmi,qmax,si,qq,cqq,cqqr,
cqqb,sq,tit,_cr,_graph,_quick;


/******************************************************
Control Parameters, may be changed  */

_cr = .95;        @ This is the confidence level used to plot the
                    critical value in the graph.  It is not used
                    elsewhere in the analysis.                          @
_graph   = 1;     @ Graph indicator
                    Set _graph=0 to not view the graph of the likelihood
                    Set _graph=1 to view the graph of the likelihood    @
_quick   = 1;     @ Indicator of method used for bootstrap
                    Set _quick=1 for efficient, quick computation
                    Set _quick=0 if memory is limited (perhaps for
                    large data sets).                                   @ 


/******************************************************/

n  = rows(dat);
q = dat[.,qi];
qs = sortind(q);
y = dat[qs,yi];
x = ones(n,1)~dat[qs,xi];
q = q[qs];
k = cols(x);
qs = unique(q,1);
qn = rows(qs);
qq = zeros(qn,1);
r=1; do while r<=qn;
  qq[r] = sumc(q.==qs[r]);
r=r+1; endo;
cqq = cumsumc(qq);
sq = (cqq.>=floor(n*trim_per)).*(cqq.<=(floor(n*(1-trim_per))));
qs = selif(qs,sq);
cqq = selif(cqq,sq);
qn = rows(qs);

mi = invpd(moment(x,0));
e  = y-x*mi*(x'y);
ee = e'e;
xe = cumsumc(x.*e);
sn = zeros(qn,1);

if _quick == 1;
  mmistore = zeros(k*(k+1)/2,qn);
  cqqb = 1;
  mm = zeros(k,k);
  r=1; do while r<=qn;
    cqqr = cqq[r];
    mm = mm + moment(x[cqqb:cqqr,.],0);
    sume = xe[cqqr,.]';
    mmi = invpd(mm - mm*mi*mm);
    sn[r] = sume'mmi*sume;
    cqqb = cqqr+1;
    mmistore[.,r] = vech(mmi);
  r=r+1;endo;
  si = maxindc(sn);
  smax = sn[si];
  qmax = qs[si];
  sig = (ee - smax)/n;
  lr = sn/sig;
  ftest = smax/sig;
  fboot = zeros(rep,1);
  j=1; do while j<=rep;
    y  = rndn(n,1);
    e  = y-x*mi*(x'y);
    ee = e'e;
    xe = cumsumc(x.*e);
    sn = zeros(qn,1);
    r=1; do while r<=qn;
      mmi = xpnd(mmistore[.,r]);
      sume = xe[cqq[r],.]';
      sn[r] = sume'mmi*sume;
    r=r+1;endo;
    smax = maxc(sn);
    sig = (ee - smax)/n;
    fboot[j] = smax/sig;
  j=j+1;endo;
elseif _quick==0;
  cqqb = 1;
  mm = zeros(k,k);
  r=1; do while r<=qn;
    cqqr = cqq[r];
    mm = mm + moment(x[cqqb:cqqr,.],0);
    sume = xe[cqqr,.]';
    mmi = mm - mm*mi*mm;
    sn[r] = sume'(sume/mmi);
    cqqb = cqqr+1;
  r=r+1;endo;
  si = maxindc(sn);
  smax = sn[si];
  qmax = qs[si];
  sig = (ee - smax)/n;
  lr = sn/sig;
  ftest = smax/sig;
  fboot = zeros(rep,1);
  j=1; do while j<=rep;
    y  = rndn(n,1);
    e  = y-x*mi*(x'y);
    ee = e'e;
    xe = cumsumc(x.*e);
    sn = zeros(qn,1);
    cqqb = 1;
    mm = zeros(k,k);
    r=1; do while r<=qn;
      cqqr = cqq[r];
      mm = mm + moment(x[cqqb:cqqr,.],0);
      mmi = mm - mm*mi*mm;
      sume = xe[cqq[r],.]';
      sn[r] = sume'(sume/mmi);
      cqqb = cqqr+1;
    r=r+1;endo;
    smax = maxc(sn);
    sig = (ee - smax)/n;
    fboot[j] = smax/sig;
  j=j+1;endo;
endif;
fboot = sortc(fboot,1);
pv = meanc(fboot .>= ftest);
cr = fboot[round(rep*_cr)];

if _graph==1;
 _pdate = "";
 fonts("complex simgrma");
 tit = "F Test For Threshold\L";
 tit = tit $+ "Reject Linearity if F Sequence Exceeds Critical Value";
 title(tit);
 ylabel("F]n[(\202g\201)");
 xlabel("\202g\201");
 clr = ones(qn,1)*cr;
 _plegstr="" $+ "LR]n[(\202g\201)\000" $+ ftocv(_cr*100,2,0)
 $+ "% Critical";
 _plegctl = 1;
 xy(qs,lr~clr);
endif;

"";"";
"Test of Null of No Threshold Against Alternative of Threshold";
"Under Maintained Assumption of Homoskedastic Errors";
"";
"Number of Bootstrap Replications " rep;
"Trimming Percentage              " trim_per;
"";
"Threshold Estimate               " qmax;
"F-test for no threshold          " ftest;
"Bootstrap P-Value                " pv;
"";"";

retp(ftest,pv);
endp;

